@@ -221,8 +221,17 @@ fn main() -> Result<()> {
         Some(msg_args::Cmd::Move(seq, mbox_dst)) => {
             return msg_handlers::move_(seq, mbox, mbox_dst, &mut printer, backend);
         }
-        Some(msg_args::Cmd::Read(seq, text_mime, raw)) => {
-            return msg_handlers::read(seq, text_mime, raw, mbox, &mut printer, backend);
+        Some(msg_args::Cmd::Read(seq, text_mime, raw, headers)) => {
+            return msg_handlers::read(
+                seq,
+                text_mime,
+                raw,
+                headers,
+                mbox,
+                &account_config,
+                &mut printer,
+                backend,
+            );
         }
         Some(msg_args::Cmd::Reply(seq, all, attachment_paths, encrypt)) => {
             return msg_handlers::reply(
